@@ -109,7 +109,7 @@ import glue.lal
 import lal
 import lalsimulation
 import lal.series
-from lalinference.bayestar.ligolw import InspiralCoincDef
+from lalinspiral.thinca import InspiralCoincDef
 
 # BAYESTAR imports.
 from lalinference.bayestar import timing
