@@ -21,21 +21,19 @@ Functions that support the command line interface.
 
 from __future__ import print_function
 import argparse
-import contextlib
 from distutils.dir_util import mkpath
 from distutils.errors import DistutilsFileError
-import errno
 import glob
 import inspect
 import itertools
 import logging
 import os
-import shutil
 import sys
 import tempfile
 import matplotlib
 from matplotlib import cm
 from ..plot import cmap
+from ..util import sqlite
 
 
 # Set no-op Matplotlib backend to defer importing anything that requires a GUI
@@ -54,14 +52,9 @@ import distutils.version
 mpl_version = distutils.version.LooseVersion(matplotlib.__version__)
 
 
-@contextlib.contextmanager
-def TemporaryDirectory(suffix='', prefix='tmp', dir=None, delete=True):
-    try:
-        dir = tempfile.mkdtemp(suffix=suffix, prefix=prefix, dir=dir)
-        yield dir
-    finally:
-        if delete:
-            shutil.rmtree(dir)
+def get_version():
+    from .. import InferenceVCSInfo as vcs_info
+    return vcs_info.name + ' ' + vcs_info.version
 
 
 class GlobAction(argparse._StoreAction):
@@ -169,7 +162,16 @@ class MatplotlibFigureType(argparse.FileType):
 
     def __save(self):
         from matplotlib import pyplot as plt
-        return plt.savefig(self.string)
+        _, ext = os.path.splitext(self.string)
+        ext = ext.lower()
+        program, _ = os.path.splitext(os.path.basename(sys.argv[0]))
+        cmdline = ' '.join([program] + sys.argv[1:])
+        metadata = {'Title': cmdline}
+        if ext == '.png':
+            metadata['Software'] = get_version()
+        elif ext in {'.pdf', '.ps', '.eps'}:
+            metadata['Creator'] = get_version()
+        return plt.savefig(self.string, metadata=metadata)
 
     def __call__(self, string):
         from matplotlib import pyplot as plt
@@ -284,8 +286,7 @@ del group
 # Defer loading SWIG bindings until version string is needed.
 class VersionAction(argparse._VersionAction):
     def __call__(self, parser, namespace, values, option_string=None):
-        from .. import InferenceVCSInfo
-        self.version = 'LALInference ' + InferenceVCSInfo.version
+        self.version = get_version()
         super(VersionAction, self).__call__(
             parser, namespace, values, option_string)
 
@@ -304,8 +305,16 @@ class LogLevelAction(argparse._StoreAction):
     def __init__(
             self, option_strings, dest, nargs=None, const=None, default=None,
             type=None, choices=None, required=False, help=None, metavar=None):
-        metavar = '|'.join(
-            _ for _ in logging._levelNames.keys() if isinstance(_, str))
+        # FIXME: this broke because of internal changes in the Python standard
+        # library logging module between Python 2.7 and 3.6. We should not rely
+        # on these undocumented module variables in the first place.
+        try:
+            logging._levelNames
+        except AttributeError:
+            metavar = '|'.join(logging._levelToName.values())
+        else:
+            metavar = '|'.join(
+                _ for _ in logging._levelNames.keys() if isinstance(_, str))
         type = loglevel_type
         super(LogLevelAction, self).__init__(
             option_strings, dest, nargs=nargs, const=const, default=default,
@@ -391,57 +400,78 @@ class DirType(object):
 class SQLiteType(argparse.FileType):
     """Open an SQLite database, or fail if it does not exist.
     FIXME: use SQLite URI when we drop support for Python < 3.4.
-    See: https://docs.python.org/3.4/whatsnew/3.4.html#sqlite3"""
+    See: https://docs.python.org/3.4/whatsnew/3.4.html#sqlite3
 
-    def __init__(self, mode='r'):
-        super(SQLiteType, self).__init__(mode + 'b')
+    Here is an example of trying to open a file that does not exist for
+    reading (mode='r'). It should raise an exception:
+
+    >>> import tempfile
+    >>> filetype = SQLiteType('r')
+    >>> filename = tempfile.mktemp()
+    >>> # Note, simply check or a FileNotFound error in Python 3.
+    >>> filetype(filename)
+    Traceback (most recent call last):
+      ...
+    argparse.ArgumentTypeError: ...
+
+    If the file already exists, then it's fine:
+    >>> import sqlite3
+    >>> filetype = SQLiteType('r')
+    >>> with tempfile.NamedTemporaryFile() as f:
+    ...     with sqlite3.connect(f.name) as db:
+    ...         _ = db.execute('create table foo (bar char)')
+    ...     filetype(f.name)
+    <sqlite3.Connection object at ...>
+
+    Here is an example of opening a file for writing (mode='w'), which should
+    overwrite the file if it exists. Even if the file was not an SQLite
+    database beforehand, this should work:
+
+    >>> filetype = SQLiteType('w')
+    >>> with tempfile.NamedTemporaryFile(mode='w') as f:
+    ...     print('This is definitely not an SQLite file.', file=f)
+    ...     f.flush()
+    ...     with filetype(f.name) as db:
+    ...         db.execute('create table foo (bar char)')
+    <sqlite3.Cursor object at ...>
+
+    Here is an example of opening a file for appending (mode='a'), which should
+    NOT overwrite the file if it exists. If the file was not an SQLite database
+    beforehand, this should raise an exception.
+
+    >>> import pytest
+    >>> filetype = SQLiteType('a')
+    >>> with tempfile.NamedTemporaryFile(mode='w') as f:
+    ...     print('This is definitely not an SQLite file.', file=f)
+    ...     f.flush()
+    ...     with filetype(f.name) as db:
+    ...         db.execute('create table foo (bar char)')
+    Traceback (most recent call last):
+      ...
+    sqlite3.DatabaseError: ...
+
+    And if the database did exist beforehand, then opening for appending
+    (mode='a') should not clobber existing tables.
+
+    >>> filetype = SQLiteType('a')
+    >>> with tempfile.NamedTemporaryFile() as f:
+    ...     with sqlite3.connect(f.name) as db:
+    ...         _ = db.execute('create table foo (bar char)')
+    ...     with filetype(f.name) as db:
+    ...         db.execute('select count(*) from foo').fetchone()
+    (0,)
+    """
+
+    def __init__(self, mode):
+        if mode not in 'arw':
+            raise ValueError('Unknown file mode: {}'.format(mode))
+        self.mode = mode
 
     def __call__(self, string):
-        if string == '-':
-            raise argparse.ArgumentTypeError(
-                'Cannot open stdin/stdout as an SQLite database')
-        with super(SQLiteType, self).__call__(string):
-            import sqlite3
-            return sqlite3.connect(string)
-
-
-def sqlite_get_filename(connection):
-    """Get the name of the file associated with an SQLite connection"""
-    result = connection.execute('pragma database_list').fetchall()
-    try:
-        (_, _, filename), = result
-    except ValueError:
-        raise RuntimeError('Expected exactly one attached database')
-    return filename
-
-
-def rename(src, dst):
-    """Like os.rename(src, dst), but works across different devices because it
-    catches and handles EXDEV ('Invalid cross-device link') errors."""
-    try:
-        os.rename(src, dst)
-    except OSError as e:
-        if e.errno == errno.EXDEV:
-            dir, suffix = os.path.split(dst)
-            tmpfid, tmpdst = tempfile.mkstemp(dir=dir, suffix=suffix)
-            try:
-                os.close(tmpfid)
-                shutil.copy2(src, tmpdst)
-                os.rename(tmpdst, dst)
-            except:
-                os.remove(tmpdst)
-                raise
-        else:
-            raise
-
-
-def rm_f(filename):
-    """Remove a file, or be silent if the file does not exist, like `rm -f`."""
-    try:
-        os.remove(filename)
-    except OSError as e:
-        if e.errno != errno.ENOENT:
-            raise
+        try:
+            return sqlite.open(string, self.mode)
+        except OSError as e:
+            raise argparse.ArgumentTypeError(e)
 
 
 def _sanitize_arg_value_for_xmldoc(value):
@@ -491,3 +521,7 @@ def iterlines(file, start_message=start_msg, stop_message=stop_msg):
 
     if is_tty:
         print(stop_message, file=sys.stderr)
+
+
+from lalinference.bayestar.deprecation import warn
+warn('ligo.skymap.tool')
