@@ -75,7 +75,7 @@ def _find_table(group, tablename):
 
     Check that we can find a file by name:
     >>> import os.path
-    >>> from lalinference.bayestar.command import TemporaryDirectory
+    >>> from lalinference.util.file import TemporaryDirectory
     >>> table = Table(np.eye(3), names=['a', 'b', 'c'])
     >>> with TemporaryDirectory() as dir:
     ...     filename = os.path.join(dir, 'test.hdf5')
@@ -150,7 +150,7 @@ def read_samples(filename, path=None, tablename=POSTERIOR_SAMPLES):
 
     Test reading a file written using the Python API:
     >>> import os.path
-    >>> from lalinference.bayestar.command import TemporaryDirectory
+    >>> from lalinference.util.file import TemporaryDirectory
     >>> table = Table([
     ...     Column(np.ones(10), name='foo', meta={'vary': FIXED}),
     ...     Column(np.arange(10), name='bar', meta={'vary': LINEAR}),
@@ -227,7 +227,7 @@ def write_samples(table, filename, metadata=None, **kwargs):
 
     And now try writing an arbitrary example to a temporary file.
     >>> import os.path
-    >>> from lalinference.bayestar.command import TemporaryDirectory
+    >>> from lalinference.util.file import TemporaryDirectory
     >>> table = Table([
     ...     Column(np.ones(10), name='foo', meta={'vary': FIXED}),
     ...     Column(np.arange(10), name='bar', meta={'vary': LINEAR}),
@@ -241,6 +241,13 @@ def write_samples(table, filename, metadata=None, **kwargs):
     # Copy the table so that we do not modify the original.
     table = table.copy()
 
+    # Make sure that all tables have a 'vary' type.
+    for column in table.columns.values():
+        if 'vary' not in column.meta:
+            if np.all(column[0] == column[1:]):
+                column.meta['vary'] = FIXED
+            else:
+                column.meta['vary'] = OUTPUT
     # Reconstruct table attributes.
     for colname, column in tuple(table.columns.items()):
         if column.meta['vary'] == FIXED:
@@ -263,3 +270,6 @@ def write_samples(table, filename, metadata=None, **kwargs):
                         raise KeyError(
                             'Unable to set metadata {0}[{1}] = {2}'.format(
                                 internal_path, key, value))
+
+from lalinference.bayestar.deprecation import warn
+warn('ligo.skymap.io.hdf5')
