@@ -66,27 +66,27 @@ def draw_posterior_many(datas, Nlives, verbose=False):
     # list of log_evidences, log_weights
     import astropy
     log_evs,log_wts=zip(*[compute_weights(data['logL'],Nlive) for data,Nlive in zip(datas, Nlives)])
-    if verbose: print 'Computed log_evidences: %s'%(str(log_evs))
+    if verbose: print('Computed log_evidences: %s'%(str(log_evs)))
 
     log_total_evidence=reduce(logaddexp, log_evs)
     log_max_evidence=max(log_evs)
     #print 'evidences: %s'%(str(log_evs))
     fracs=[exp(log_ev-log_max_evidence) for log_ev in log_evs]
-    if verbose: print 'Relative weights of input files: %s'%(str(fracs))
+    if verbose: print('Relative weights of input files: %s'%(str(fracs)))
     Ns=[fracs[i]/len(datas[i]) for i in range(len(fracs))]
     Ntot=max(Ns)
     fracs=[n/Ntot for n in Ns]
-    if verbose: print 'Relative weights of input files taking into account their length: %s'%(str(fracs))
+    if verbose: print('Relative weights of input files taking into account their length: %s'%(str(fracs)))
 
     posts=[draw_posterior(data,logwt) for (data,logwt,logZ) in zip(datas,log_wts,log_evs)]
-    if verbose: print 'Number of input samples: %s'%(str([len(x) for x in log_wts]))
-    if verbose: print 'Expected number of samples from each input file %s'%(str([int(f*len(p)) for f,p in zip(fracs,posts)]))
+    if verbose: print('Number of input samples: %s'%(str([len(x) for x in log_wts])))
+    if verbose: print('Expected number of samples from each input file %s'%(str([int(f*len(p)) for f,p in zip(fracs,posts)])))
     bigpos=[]
     for post,frac in zip(posts,fracs):
       mask = uniform(size=len(post))<frac
       bigpos.append(post[mask])
     result = astropy.table.vstack(bigpos)
-    if verbose: print 'Total number of samples produced: %i'%(len(result))
+    if verbose: print('Total number of samples produced: %i'%(len(result)))
     return result
     
 def draw_N_posterior(data,log_wts, N, verbose=False):
@@ -131,21 +131,21 @@ def draw_posterior_many_ROQ_runs(datas, Nlives, verbose=False):
     # list of log_evidences, log_weights
     import astropy
     log_evs,log_wts=zip(*[compute_weights(data['logL'],Nlive) for data,Nlive in zip(datas, Nlives)])
-    if verbose: print 'Computed log_evidences: %s'%(str(log_evs))
+    if verbose: print('Computed log_evidences: %s'%(str(log_evs)))
 
     log_total_evidence=reduce(logaddexp, log_evs)
     log_max_evidence=max(log_evs)
     #print 'evidences: %s'%(str(log_evs))
     fracs=[exp(log_ev-log_max_evidence) for log_ev in log_evs] #TODO: add logPriorVol from ROQ run
-    if verbose: print 'Relative weights of input files: %s'%(str(fracs))
+    if verbose: print('Relative weights of input files: %s'%(str(fracs)))
     Ns=[fracs[i]/len(datas[i]) for i in range(len(fracs))]
     Ntot=max(Ns)
     fracs=[n/Ntot for n in Ns]
-    if verbose: print 'Relative weights of input files taking into account their length: %s'%(str(fracs))
+    if verbose: print('Relative weights of input files taking into account their length: %s'%(str(fracs)))
 
     bigpos=[]
     posts=[draw_posterior(data,logwt) for (data,logwt,logZ) in zip(datas,log_wts,log_evs)]
-    if verbose: print 'Number of input samples: %s'%(str([len(x) for x in log_wts]))
-    if verbose: print 'Expected number of samples from each input file %s'%(str([int(f*len(p)) for f,p in zip(fracs,posts)]))
+    if verbose: print('Number of input samples: %s'%(str([len(x) for x in log_wts])))
+    if verbose: print('Expected number of samples from each input file %s'%(str([int(f*len(p)) for f,p in zip(fracs,posts)])))
     return astropy.table.vstack(bigpos)
  
