@@ -17,11 +17,16 @@
 """
 A collection of useful data analysis routines built from the SWIG wrappings of LAL and LALSimulation.
 """
+
+from __future__ import print_function
+
 import sys
 import copy
 import types
 from math import cos, sin, sqrt
 
+from six.moves import range
+
 import numpy as np
 from scipy import interpolate
 from scipy import signal
@@ -109,6 +114,9 @@ class ChooseWaveformParams:
         for k, p in ChooseWaveformParams._LAL_DICT_PARAMS.iteritems():
             typfunc = ChooseWaveformParams._LAL_DICT_PTYPE[k]
             typfunc(extra_params, k, getattr(self, p))
+        # Properly add tidal parammeters
+        lalsim.SimInspiralWaveformParamsInsertTidalLambda1(extra_params, self.lambda1)
+        lalsim.SimInspiralWaveformParamsInsertTidalLambda2(extra_params, self.lambda2)
         return extra_params
 
     def copy(self):
@@ -121,52 +129,52 @@ class ChooseWaveformParams:
         """
         Print all key-value pairs belonging in the class instance
         """
-        print "This ChooseWaveformParams has the following parameter values:"
-        print "m1 =", self.m1 / lal.MSUN_SI, "(Msun)"
-        print "m2 =", self.m2 / lal.MSUN_SI, "(Msun)"
-        print "spin1x =", self.spin1x
-        print "spin1y =", self.spin1y
-        print "spin1z =", self.spin1z
-        print "spin2x =", self.spin2x
-        print "spin2y =", self.spin2y
-        print "spin2z =", self.spin2z
-        print "lambda1 =", self.lambda1
-        print "lambda2 =", self.lambda2
-        print "inclination =", self.incl
-        print "distance =", self.dist / 1.e+6 / lal.PC_SI, "(Mpc)"
-        print "reference orbital phase =", self.phiref
-        print "time of coalescence =", float(self.tref)
-        print "detector is:", self.detector
+        print("This ChooseWaveformParams has the following parameter values:")
+        print("m1 =", self.m1 / lal.MSUN_SI, "(Msun)")
+        print("m2 =", self.m2 / lal.MSUN_SI, "(Msun)")
+        print("spin1x =", self.spin1x)
+        print("spin1y =", self.spin1y)
+        print("spin1z =", self.spin1z)
+        print("spin2x =", self.spin2x)
+        print("spin2y =", self.spin2y)
+        print("spin2z =", self.spin2z)
+        print("lambda1 =", self.lambda1)
+        print("lambda2 =", self.lambda2)
+        print("inclination =", self.incl)
+        print("distance =", self.dist / 1.e+6 / lal.PC_SI, "(Mpc)")
+        print("reference orbital phase =", self.phiref)
+        print("time of coalescence =", float(self.tref))
+        print("detector is:", self.detector)
         if self.radec==False:
-            print "Sky position relative to overhead detector is:"
-            print "zenith angle =", self.theta, "(radians)"
-            print "azimuth angle =", self.phi, "(radians)"
+            print("Sky position relative to overhead detector is:")
+            print("zenith angle =", self.theta, "(radians)")
+            print("azimuth angle =", self.phi, "(radians)")
         if self.radec==True:
-            print "Sky position relative to geocenter is:"
-            print "declination =", self.theta, "(radians)"
-            print "right ascension =", self.phi, "(radians)"
-        print "polarization angle =", self.psi
-        print "starting frequency is =", self.fmin
-        print "reference frequency is =", self.fref
-        print "Max frequency is =", self.fmax
-        print "time step =", self.deltaT, "(s) <==>", 1./self.deltaT,\
-                "(Hz) sample rate"
-        print "freq. bin size is =", self.deltaF, "(Hz)"
-        print "approximant is =", lalsim.GetStringFromApproximant(self.approx)
-        print "phase order =", self.phaseO
-        print "amplitude order =", self.ampO
-        print "waveFlags struct is", self.waveFlags
-        print "nonGRparams struct is", self.nonGRparams
+            print("Sky position relative to geocenter is:")
+            print("declination =", self.theta, "(radians)")
+            print("right ascension =", self.phi, "(radians)")
+        print("polarization angle =", self.psi)
+        print("starting frequency is =", self.fmin)
+        print("reference frequency is =", self.fref)
+        print("Max frequency is =", self.fmax)
+        print("time step =", self.deltaT, "(s) <==>", 1./self.deltaT,\
+                "(Hz) sample rate")
+        print("freq. bin size is =", self.deltaF, "(Hz)")
+        print("approximant is =", lalsim.GetStringFromApproximant(self.approx))
+        print("phase order =", self.phaseO)
+        print("amplitude order =", self.ampO)
+        print("waveFlags struct is", self.waveFlags)
+        print("nonGRparams struct is", self.nonGRparams)
         if self.taper==lalsim.SIM_INSPIRAL_TAPER_NONE:
-            print "Tapering is set to SIM_INSPIRAL_TAPER_NONE"
+            print("Tapering is set to SIM_INSPIRAL_TAPER_NONE")
         elif self.taper==lalsim.SIM_INSPIRAL_TAPER_START:
-            print "Tapering is set to SIM_INSPIRAL_TAPER_START"
+            print("Tapering is set to SIM_INSPIRAL_TAPER_START")
         elif self.taper==lalsim.SIM_INSPIRAL_TAPER_END:
-            print "Tapering is set to SIM_INSPIRAL_TAPER_END"
+            print("Tapering is set to SIM_INSPIRAL_TAPER_END")
         elif self.taper==lalsim.SIM_INSPIRAL_TAPER_STARTEND:
-            print "Tapering is set to SIM_INSPIRAL_TAPER_STARTEND"
+            print("Tapering is set to SIM_INSPIRAL_TAPER_STARTEND")
         else:
-            print "Warning! Invalid value for taper:", self.taper
+            print("Warning! Invalid value for taper:", self.taper)
 
     def copy_sim_inspiral(self, row):
         """
@@ -262,7 +270,7 @@ def xml_to_ChooseWaveformParams_array(fname, minrow=None, maxrow=None,
         # Copy the information from requested rows to the ChooseWaveformParams
         [Ps[i-minrow].copy_lsctables_sim_inspiral(sim_insp[i]) for i in rng]
     except ValueError:
-        print >>sys.stderr, "No SimInspiral table found in xml file"
+        print("No SimInspiral table found in xml file", file=sys.stderr)
     return Ps
 
 
@@ -329,7 +337,7 @@ class InnerProduct(object):
             WFD.data.data[:] = np.sqrt(self.weights) # W_FD is 1/sqrt(S_n(f))
             WFD.data.data[0] = WFD.data.data[-1] = 0. # zero 0, f_Nyq bins
             lal.REAL8FreqTimeFFT(WTD, WFD, revplan) # IFFT to TD
-            for i in xrange(N_spec/2, self.len2side - N_spec/2):
+            for i in range(N_spec/2, self.len2side - N_spec/2):
                 WTD.data.data[i] = 0. # Zero all but T_spec/2 ends of W_TD
             lal.REAL8TimeFreqFFT(WFD, WTD, fwdplan) # FFT back to FD
             WFD.data.data[0] = WFD.data.data[-1] = 0. # zero 0, f_Nyq bins
@@ -539,7 +547,7 @@ class Overlap(InnerProduct):
         Compute norm of a COMPLEX16Frequency Series
         """
         if h.data.length != self.len1side:
-            print "Bad data length, needed %d, got %d" % (self.len1side, h.data.length)
+            print("Bad data length, needed %d, got %d" % (self.len1side, h.data.length))
         assert h.data.length == self.len1side
         assert abs(h.deltaF-self.deltaF) <= TOL_DF
         val = 0.
@@ -770,6 +778,7 @@ def Mceta(m1, m2):
 def tidal_lambda_tilde(mass1, mass2, lambda1, lambda2):
     """
     'Effective' lambda parameters.
+    See https://arxiv.org/pdf/1402.5156.pdf
     """
     mt = mass1 + mass2
     eta = mass1 * mass2 / mt**2
@@ -778,7 +787,7 @@ def tidal_lambda_tilde(mass1, mass2, lambda1, lambda2):
     lt_sym = lt1 + lt2
     lt_asym = lt1 - lt2
 
-    lam_til = (1 + 7*eta - 31*eta**2) * lt_sym - q * (1 + 9*eta - 11*eta**2) * lt_asym
+    lam_til = (1 + 7*eta - 31*eta**2) * lt_sym + q * (1 + 9*eta - 11*eta**2) * lt_asym
     dlam_til = q * (1 - 13272*eta/1319 + 8944*eta**2/1319) * lt_sym + (1 - 15910*eta/1319 + 32850*eta**2/1319 + 3380*eta**3/1319) * lt_asym
     dlam_til *= 0.5
     lam_til *= 8. / 13
@@ -987,63 +996,7 @@ def hoff(P, Fp=None, Fc=None):
         htilde = lal.ResizeCOMPLEX16FrequencySeries(htilde, 0, FDlen)
     return htilde
 
-def norm_hoff(P, IP, Fp=None, Fc=None, fwdplan=None):
-    """
-    Generate a normalized FD waveform from ChooseWaveformParams P.
-    Will return a COMPLEX16FrequencySeries object.
-
-    If P.approx is a FD approximant, norm_hoff_FD is called.
-    This path calls SimInspiralChooseFDWaveform
-        fwdplan must be None for FD approximants.
-
-    If P.approx is a TD approximant, norm_hoff_TD is called.
-    This path calls ChooseTDWaveform and performs an FFT.
-        The TD waveform will be zero-padded so it's Fourier transform has
-        frequency bins of size P.deltaT.
-        If P.deltaF == None, the TD waveform will be zero-padded
-        to the next power of 2.
-    """
-    # For FD approximants, use the ChooseFDWaveform path = hoff_FD
-    if lalsim.SimInspiralImplementedFDApproximants(P.approx)==1:
-        # Raise exception if unused arguments were specified
-        if fwdplan is not None:
-            raise ValueError('FFT plan fwdplan given with FD approximant.\nFD approximants cannot use this.')
-        hf = norm_hoff_FD(P, IP, Fp, Fc)
-
-    # For TD approximants, do ChooseTDWaveform + FFT path = hoff_TD
-    else:
-        hf = norm_hoff_TD(P, IP, Fp, Fc, fwdplan)
-
-    return hf
-
-def norm_hoff_TD(P, IP, Fp=None, Fc=None, fwdplan=None):
-    """
-    Generate a waveform from ChooseWaveformParams P normalized according
-    to inner product IP by creating a TD waveform, zero-padding and
-    then Fourier transforming with FFTW3 forward FFT plan fwdplan.
-    Returns a COMPLEX16FrequencySeries object.
-
-    If P.deltaF==None, just pad up to next power of 2
-    If P.deltaF = 1/X, will generate a TD waveform, zero-pad to length X seconds
-        and then FFT. Will throw an error if waveform is longer than X seconds
-
-    If you do not provide a forward FFT plan, one will be created.
-    If you are calling this function many times, you may to create it
-    once beforehand and pass it in, e.g.:
-    fwdplan=lal.CreateForwardREAL8FFTPlan(TDlen,0)
-
-    You may pass in antenna patterns Fp, Fc. If none are provided, they will
-    be computed from the information in ChooseWaveformParams.
-
-    N.B. IP and the waveform generated from P must have the same deltaF and 
-        the waveform must extend to at least the highest frequency of IP's PSD.
-    """
-    hf = hoff_TD(P, Fp, Fc, fwdplan)
-    norm = IP.norm(hf)
-    hf.data.data /= norm
-    return hf
-
-def norm_hoff_FD(P, IP, Fp=None, Fc=None):
+def norm_hoff(P, IP, Fp=None, Fc=None):
     """
     Generate a FD waveform for a FD approximant normalized according to IP.
     Note that P.deltaF (which is None by default) must be set.
@@ -1053,7 +1006,7 @@ def norm_hoff_FD(P, IP, Fp=None, Fc=None):
     if P.deltaF is None:
         raise ValueError('None given for freq. bin size P.deltaF')
 
-    htilde = hoff_FD(P, Fp, Fc)
+    htilde = hoff(P, Fp, Fc)
     norm = IP.norm(htilde)
     htilde.data.data /= norm
     return htilde
@@ -1082,8 +1035,6 @@ def non_herm_hoff(P):
     lal.COMPLEX16TimeFreqFFT(hf, htC, fwdplan)
     return hf
 
-
-
 def hlmoft(P, Lmax=2, Fp=None, Fc=None):
     """
     Generate the TD h_lm -2-spin-weighted spherical harmonic modes of a GW
@@ -1101,6 +1052,8 @@ def hlmoft(P, Lmax=2, Fp=None, Fc=None):
         hlms = lalsim.SimInspiralChooseTDModes( \
             P.phiref, P.deltaT, \
             P.m1, P.m2, \
+            P.spin1x, P.spin1y, P.spin1z, \
+            P.spin2x, P.spin2y, P.spin2z, \
             P.fmin, P.fref, P.dist, \
             extra_params, Lmax, P.approx)
     except RuntimeError:
@@ -1108,7 +1061,7 @@ def hlmoft(P, Lmax=2, Fp=None, Fc=None):
             P.m1, P.m2, \
             P.spin1x, P.spin1y, P.spin1z, \
             P.spin2x, P.spin2y, P.spin2z, \
-            P.dist, P.incl, P.phiref,  \
+            P.dist, P.phiref,  \
             P.psi, P.eccentricity, P.meanPerAno, \
             P.deltaT, P.fmin, P.fref, \
             extra_params, P.approx)
@@ -1261,7 +1214,6 @@ def complex_hoff(P, sgn=-1, fwdplan=None):
 
     Returns a COMPLEX16FrequencySeries object
     """
-    ht = complex_hoft(P, sgn)
     assert sgn == 1 or sgn == -1
 
     extra_params = P.to_lal_dict()
@@ -1272,7 +1224,7 @@ def complex_hoff(P, sgn=-1, fwdplan=None):
             P.spin2x, P.spin2y, P.spin2z, \
             P.dist, P.incl, P.phiref,  \
             P.psi, P.eccentricity, P.meanPerAno, \
-            P.deltaT, P.fmin, P.fref, \
+            P.deltaF, P.fmin, P.fmax, P.fref, \
             extra_params, P.approx)
 
     # N.B. TaylorF2(RedSpin)(Tidal)  stop filling the output array at ISCO.
@@ -1327,7 +1279,7 @@ def frame_data_to_hoft(fname, channel, start=None, stop=None, window_shape=0.,
             strength over that fraction of each end of the data segment.
     """
     if verbose:
-        print " ++ Loading from cache ", fname, channel
+        print(" ++ Loading from cache ", fname, channel)
     with open(fname) as cfile:
         cachef = Cache.fromfile(cfile)
     for i in range(len(cachef))[::-1]:
@@ -1335,7 +1287,7 @@ def frame_data_to_hoft(fname, channel, start=None, stop=None, window_shape=0.,
         if cachef[i].observatory != channel[0]:
             del cachef[i]
     if verbose:
-        print cachef.to_segmentlistdict()
+        print(cachef.to_segmentlistdict())
     fcache = frutils.FrameCache(cachef)
     # FIXME: Horrible, horrible hack -- will only work if all requested channels
     # span the cache *exactly*
@@ -1349,7 +1301,7 @@ def frame_data_to_hoft(fname, channel, start=None, stop=None, window_shape=0.,
     tmp = lal.CreateREAL8TimeSeries("h(t)", 
             lal.LIGOTimeGPS(float(ht.metadata.segments[0][0])),
             0., ht.metadata.dt, lal.DimensionlessUnit, len(ht))
-    print   "  ++ Frame data sampling rate ", 1./tmp.deltaT, " and epoch ", string_gps_pretty_print(tmp.epoch)
+    print("  ++ Frame data sampling rate ", 1./tmp.deltaT, " and epoch ", string_gps_pretty_print(tmp.epoch))
     tmp.data.data[:] = ht
     # Window the data - N.B. default is identity (no windowing)
     hoft_window = lal.CreateTukeyREAL8Window(tmp.data.length, window_shape)
@@ -1437,7 +1389,7 @@ def frame_data_to_non_herm_hoff(fname, channel, start=None, stop=None, TDlen=0,
             FDlen)
     lal.COMPLEX16TimeFreqFFT(hf, hoftC, fwdplan)
     if verbose:
-        print " ++ Loaded data h(f) of length n= ", hf.data.length, " (= ", len(hf.data.data)*ht.deltaT, "s) at sampling rate ", 1./ht.deltaT    
+        print(" ++ Loaded data h(f) of length n= ", hf.data.length, " (= ", len(hf.data.data)*ht.deltaT, "s) at sampling rate ", 1./ht.deltaT)    
     return hf
 
 
@@ -1520,13 +1472,13 @@ if __name__ == "__main__":
         try:
             hoft(params)
         except RuntimeError:
-            print >>sys.stderr, "TD waveform generation failed for params\n" \
-                "masses: %e %e, spins: %e, %e, approx: %s" % (m1i, m2i, s1zi, s2zi, a)
+            print("TD waveform generation failed for params\n" \
+                "masses: %e %e, spins: %e, %e, approx: %s" % (m1i, m2i, s1zi, s2zi, a), file=sys.stderr)
 
         params.deltaF = 1.0/8
         params.fmax = 2048.
         try:
             hoff(params)
         except RuntimeError:
-            print >>sys.stderr, "FD waveform generation failed for params\n" \
-                "masses: %e %e, spins: %e, %e, approx: %s" % (m1i, m2i, s1zi, s2zi, a)
+            print("FD waveform generation failed for params\n" \
+                "masses: %e %e, spins: %e, %e, approx: %s" % (m1i, m2i, s1zi, s2zi, a), file=sys.stderr)
