@@ -81,9 +81,9 @@ static int get_calib_spline(LALInferenceVariables *vars, const char *ifoname, RE
 
   for(UINT4 i=0;i<npts;i++)
   {
-    snprintf(freqname, VARNAME_MAX, "%s_spcal_logfreq_%i", ifoname, i);
-    snprintf(ampname, VARNAME_MAX, "%s_spcal_amp_%i", ifoname, i);
-    snprintf(phasename, VARNAME_MAX, "%s_spcal_phase_%i", ifoname, i);
+    if((VARNAME_MAX <= snprintf(freqname, VARNAME_MAX, "%s_spcal_logfreq_%i", ifoname, i))) XLAL_ERROR(XLAL_EINVAL,"Variable name too long");
+    if((VARNAME_MAX <= snprintf(ampname, VARNAME_MAX, "%s_spcal_amp_%i", ifoname, i))) XLAL_ERROR(XLAL_EINVAL,"Variable name too long");
+    if((VARNAME_MAX <= snprintf(phasename, VARNAME_MAX, "%s_spcal_phase_%i", ifoname, i))) XLAL_ERROR(XLAL_EINVAL,"Variable name too long");
 
     (*logfreqs)->data[i] = LALInferenceGetREAL8Variable(vars, freqname);
     (*amps)->data[i] =  LALInferenceGetREAL8Variable(vars, ampname);
@@ -176,15 +176,14 @@ void LALInferenceInitLikelihood(LALInferenceRunState *runState)
    }
 
    /* Try to determine a model-less likelihood, if such a thing makes sense */
-   if (runState->likelihood==&LALInferenceUndecomposedFreqDomainLogLikelihood){
+   if (runState->likelihood==&LALInferenceUndecomposedFreqDomainLogLikelihood || runState->likelihood==&LALInferenceMarginalisedPhaseLogLikelihood ){
 
                 nullLikelihood = LALInferenceNullLogLikelihood(runState->data);
 
     }
     else if (runState->likelihood==&LALInferenceFreqDomainStudentTLogLikelihood ||
        runState->likelihood==&LALInferenceMarginalisedTimeLogLikelihood ||
-       runState->likelihood==&LALInferenceMarginalisedTimePhaseLogLikelihood ||
-       runState->likelihood==&LALInferenceMarginalisedPhaseLogLikelihood) {
+       runState->likelihood==&LALInferenceMarginalisedTimePhaseLogLikelihood) {
 
        void *oldtemplate = runState->threads[0]->model->templt;
        if (runState->threads[0]->model->domain == LAL_SIM_DOMAIN_FREQUENCY) {
@@ -395,6 +394,9 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
   if (marginalisationflags==MARGTIME || marginalisationflags==MARGTIMEPHI)
     margtime=1;
 
+  if(model->roq_flag && margtime) XLAL_ERROR_REAL8(XLAL_EINVAL,"ROQ does not support time marginalisation");
+
+  
   LALStatus status;
   memset(&status,0,sizeof(status));
 
@@ -483,7 +485,6 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
     INT4 SKY_FRAME=0;
     if(LALInferenceCheckVariable(currentParams,"SKY_FRAME"))
       SKY_FRAME=*(INT4 *)LALInferenceGetVariable(currentParams,"SKY_FRAME");
-
     if(SKY_FRAME==0){
       /* determine source's sky location & orientation parameters: */
       ra        = *(REAL8*) LALInferenceGetVariable(currentParams, "rightascension"); /* radian      */
@@ -582,7 +583,7 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
     if(marginalisationflags==STUDENTT)
     {
       /* extract the element from the "df" vector that carries the current Ifo's name: */
-      CHAR df_variable_name[64];
+      CHAR df_variable_name[320];
       snprintf(df_variable_name,sizeof(df_variable_name),"df_%s",dataPtr->name);
       if(LALInferenceCheckVariable(currentParams,df_variable_name)){
         printf("Found variable %s\n",df_variable_name);
@@ -625,6 +626,18 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
           switch(errnum)
           {
             case XLAL_EUSR0: /* Template generation failed in a known way, set -Inf likelihood */
+		      /* Free up allocated vectors */
+              if(dh_S_tilde) XLALDestroyCOMPLEX16Vector(dh_S_tilde);
+              if(dh_S) XLALDestroyREAL8Vector(dh_S);
+              if(dh_S_phase_tilde) XLALDestroyCOMPLEX16Vector(dh_S_phase_tilde);
+              if(dh_S_phase) XLALDestroyREAL8Vector(dh_S_phase);
+              if(model->roq_flag)
+              {
+                if ( model->roq->hptildeLinear ) XLALDestroyCOMPLEX16FrequencySeries(model->roq->hptildeLinear);
+                if ( model->roq->hctildeLinear ) XLALDestroyCOMPLEX16FrequencySeries(model->roq->hctildeLinear);
+                if ( model->roq->hptildeQuadratic ) XLALDestroyCOMPLEX16FrequencySeries(model->roq->hptildeQuadratic);
+                if ( model->roq->hctildeQuadratic ) XLALDestroyCOMPLEX16FrequencySeries(model->roq->hctildeQuadratic);
+              }
               return (-INFINITY);
               break;
             default: /* Panic! */
@@ -678,13 +691,13 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
         }
         /*constant*/
         if (constantcal_active){
-          char CA_A[10]="";
+          char CA_A[320];
           sprintf(CA_A,"%s_%s","calamp",dataPtr->name);
           if (LALInferenceCheckVariable(currentParams, CA_A))
             calamp=(*(REAL8*) LALInferenceGetVariable(currentParams, CA_A));
           else
             calamp=0.0;
-          char CP_A[10]="";
+          char CP_A[320];
           sprintf(CP_A,"%s_%s","calpha",dataPtr->name);
           if (LALInferenceCheckVariable(currentParams, CP_A))
             calpha=(*(REAL8*) LALInferenceGetVariable(currentParams, CP_A));
@@ -803,24 +816,54 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
 	}
 
 	d_inner_h += creal(this_ifo_d_inner_h);
-	S += this_ifo_s;
+        // D gets the factor of 2 inside nullloglikelihood
+        // R gets a factor of 2 later, before entering the Bessel function
+	S += 0.5*this_ifo_s;
+        D += -dataPtr->nullloglikelihood;
+        Rcplx += 0.5*this_ifo_d_inner_h;
+
+        
 	model->ifo_loglikelihoods[ifo] = creal(this_ifo_d_inner_h) - (0.5*this_ifo_s) + dataPtr->nullloglikelihood;
-
-	loglikelihood += model->ifo_loglikelihoods[ifo];
-
+    
+    switch(marginalisationflags)
+    {
+    case GAUSSIAN:
+    case STUDENTT:
+      loglikelihood += model->ifo_loglikelihoods[ifo];
+      break;
+    case MARGTIME:
+    case MARGPHI:
+    case MARGTIMEPHI:
+      /* These are non-separable likelihoods, so single IFO log(L)
+	 doesn't make sense. */
+      model->ifo_loglikelihoods[ifo] = 0.0;
+      break;
+    default:
+      break;
+    }
+    
 	char varname[VARNAME_MAX];
-    	sprintf(varname,"%s_optimal_snr",dataPtr->name);
-    	REAL8 this_ifo_snr = sqrt(this_ifo_s);
-	model->ifo_SNRs[ifo] = this_ifo_snr;
-    	LALInferenceAddREAL8Variable(currentParams,varname,this_ifo_snr,LALINFERENCE_PARAM_OUTPUT);
+    if((VARNAME_MAX <= snprintf(varname,VARNAME_MAX,"%s_optimal_snr",dataPtr->name)))
+    {
+        fprintf(stderr,"variable name too long\n"); exit(1);
+    }
+    REAL8 this_ifo_snr = sqrt(this_ifo_s);
+    model->ifo_SNRs[ifo] = this_ifo_snr;
+    LALInferenceAddREAL8Variable(currentParams,varname,this_ifo_snr,LALINFERENCE_PARAM_OUTPUT);
 
-	sprintf(varname,"%s_cplx_snr_amp",dataPtr->name);
-	REAL8 cplx_snr_amp = cabs(this_ifo_d_inner_h)/this_ifo_snr;
-    	LALInferenceAddREAL8Variable(currentParams,varname,cplx_snr_amp,LALINFERENCE_PARAM_OUTPUT);
+    if((VARNAME_MAX <= snprintf(varname,VARNAME_MAX,"%s_cplx_snr_amp",dataPtr->name)))
+    {
+        fprintf(stderr,"variable name too long\n"); exit(1);
+    }
+    REAL8 cplx_snr_amp = cabs(this_ifo_d_inner_h)/this_ifo_snr;
+    LALInferenceAddREAL8Variable(currentParams,varname,cplx_snr_amp,LALINFERENCE_PARAM_OUTPUT);
 
-    	sprintf(varname,"%s_cplx_snr_arg",dataPtr->name);
-    	REAL8 cplx_snr_phase = carg(this_ifo_d_inner_h);
-    	LALInferenceAddREAL8Variable(currentParams,varname,cplx_snr_phase,LALINFERENCE_PARAM_OUTPUT);
+    if((VARNAME_MAX <= snprintf(varname,VARNAME_MAX,"%s_cplx_snr_arg",dataPtr->name)))
+    {
+        fprintf(stderr,"variable name too long\n"); exit(1);
+    }
+    REAL8 cplx_snr_phase = carg(this_ifo_d_inner_h);
+    LALInferenceAddREAL8Variable(currentParams,varname,cplx_snr_phase,LALINFERENCE_PARAM_OUTPUT);
 
 
     }
@@ -981,17 +1024,26 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
     }
     S+=this_ifo_S;
     char varname[VARNAME_MAX];
-    sprintf(varname,"%s_optimal_snr",dataPtr->name);
+    if((VARNAME_MAX <= snprintf(varname,VARNAME_MAX,"%s_optimal_snr",dataPtr->name)))
+    {
+        fprintf(stderr,"variable name too long\n"); exit(1);
+    }
     LALInferenceAddREAL8Variable(currentParams,varname,sqrt(2.0*this_ifo_S),LALINFERENCE_PARAM_OUTPUT);
 
-    sprintf(varname,"%s_cplx_snr_amp",dataPtr->name);
+    if((VARNAME_MAX <= snprintf(varname,VARNAME_MAX,"%s_cplx_snr_amp",dataPtr->name)))
+    {
+        fprintf(stderr,"variable name too long\n"); exit(1);
+    }
     REAL8 cplx_snr_amp=0.0;
     REAL8 cplx_snr_phase=carg(this_ifo_Rcplx);
     if(this_ifo_S > 0) cplx_snr_amp=2.0*cabs(this_ifo_Rcplx)/sqrt(2.0*this_ifo_S);
 
     LALInferenceAddREAL8Variable(currentParams,varname,cplx_snr_amp,LALINFERENCE_PARAM_OUTPUT);
 
-    sprintf(varname,"%s_cplx_snr_arg",dataPtr->name);
+    if((VARNAME_MAX <= snprintf(varname,VARNAME_MAX,"%s_cplx_snr_arg",dataPtr->name)))
+    {
+        fprintf(stderr,"variable name too long\n"); exit(1);
+    }
     LALInferenceAddREAL8Variable(currentParams,varname,cplx_snr_phase,LALINFERENCE_PARAM_OUTPUT);
 
    /* Clean up calibration if necessary */
@@ -1043,10 +1095,6 @@ static REAL8 LALInferenceFusedFreqDomainLogLikelihood(LALInferenceVariables *cur
 		}
         }
 
-	return(loglikelihood); /* The ROQ isn't compatible with the stuff below, so we can just exit here */
-
-
-
   }
 
   // for models which are non-factorising
