@@ -8,6 +8,8 @@
 #include <stdlib.h>
 #include <float.h>
 #include <signal.h>
+#include <time.h>
+#include <sys/times.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -19,6 +21,7 @@
 #include <lal/LALInferencePrior.h>
 #include <lal/LALInferenceLikelihood.h>
 #include <lal/LALInferenceProposal.h>
+#include <lal/LALInferenceReadData.h>
 #include <lal/LALInferenceHDF5.h>
 #include <lal/LALInferencePriorVolumes.h>
 
@@ -116,7 +119,14 @@ static int WriteNSCheckPointH5(char *filename, LALInferenceRunState *runState, N
     LALInferenceH5VariablesArrayToDataset(group, output_array, N_output_array, "past_chain");
     XLALH5FileAddScalarAttribute(group, "N_outputarray", &N_output_array, LAL_I4_TYPE_CODE);
   }
-  
+  struct tms tms_buffer;
+  if(times(&tms_buffer))
+  {
+    REAL8 execution_time = (REAL8)tms_buffer.tms_utime / (REAL8) sysconf (_SC_CLK_TCK);
+    REAL8 *prevtime=NULL;
+    if( (prevtime=LALInferenceGetVariable(runState->algorithmParams,"cpu_time"))) execution_time+=*prevtime;
+    XLALH5FileAddScalarAttribute(group, "cpu_time", &execution_time, LAL_D_TYPE_CODE);
+  }
   XLALH5FileClose(group);
   XLALH5FileClose(h5file);
   return(retcode);
@@ -161,6 +171,12 @@ static int ReadNSCheckPointH5(char *filename, LALInferenceRunState *runState, NS
     LALInferenceAddVariable(runState->algorithmParams,"outputarray",&outputarray,LALINFERENCE_void_ptr_t,LALINFERENCE_PARAM_OUTPUT);
     XLALH5DatasetFree(outputGroup);
   }
+  double execution_time = 0.0;
+  if ( ( execution_time = XLALH5FileQueryScalarAttributeValue(&execution_time, group, "cpu_time") ) )
+  {
+      LALInferenceAddVariable(runState->algorithmParams, "cpu_time", &execution_time, LALINFERENCE_REAL8_t, LALINFERENCE_PARAM_OUTPUT);
+  }
+  
   XLALH5FileClose(group);
   XLALH5FileClose(h5file);
   printf("done restoring\n");
@@ -331,6 +347,34 @@ static void catch_alarm(UNUSED int sig, UNUSED siginfo_t *siginfo,UNUSED void *c
   __ns_saveStateFlag=1;
 }
 
+static void install_resume_handler(void);
+static void install_resume_handler(void)
+{
+    /* Install a periodic alarm that will trigger a checkpoint */
+    int sigretcode=0;
+    struct sigaction sa;
+    sa.sa_sigaction=catch_alarm;
+    sa.sa_flags=SA_SIGINFO;
+    sigretcode=sigaction(SIGVTALRM,&sa,NULL);
+    if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint timer!\n");
+    /* Condor sends SIGUSR2 to checkpoint and continue */
+    sigretcode=sigaction(SIGUSR2,&sa,NULL);
+    if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGUSR2.\n");
+    checkpoint_timer.it_interval.tv_sec=30*60; /* Default timer 30 mins */
+    checkpoint_timer.it_interval.tv_usec=0;
+    checkpoint_timer.it_value=checkpoint_timer.it_interval;
+    setitimer(ITIMER_VIRTUAL,&checkpoint_timer,NULL);
+    /* Install the handler for the condor interrupt signal */
+    sa.sa_sigaction=catch_interrupt;
+    sigretcode=sigaction(SIGINT,&sa,NULL);
+    if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGINT.\n");
+    /* Condor sends SIGTERM to vanilla universe jobs to evict them */
+    sigretcode=sigaction(SIGTERM,&sa,NULL);
+    if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGTERM.\n");
+    /* Condor sends SIGTSTP to standard universe jobs to evict them.
+     *I think condor handles this, so didn't add a handler CHECK */
+}
+
 static UINT4 UpdateNMCMC(LALInferenceRunState *runState);
 /* Prototypes for private "helper" functions. */
 
@@ -750,6 +794,9 @@ void LALInferenceNestedSamplingAlgorithmInit(LALInferenceRunState *runState)
     LALInferenceAddVariable(runState->algorithmParams,"tolerance",&tmp, LALINFERENCE_REAL8_t,
                             LALINFERENCE_PARAM_FIXED);
   }
+  double zero=0.0;
+  LALInferenceAddVariable(runState->algorithmParams,"cpu_time",&zero,LALINFERENCE_REAL8_t, LALINFERENCE_PARAM_OUTPUT);
+  
   return;
 
 }
@@ -856,7 +903,13 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   if ( LALInferenceCheckVariable( runState->livePoints[0], "chirpmass" ) ){
     /* If a cbc run, calculate the mass-distance volume and store it to file*/ 
     /* Do it before algorithm starts so that we can kill the run and still get this */
-    logvolume=log(LALInferenceMassDistancePriorVolume(runState));
+    int errnum=0;
+    XLAL_TRY(logvolume=log(LALInferenceMassDistancePriorVolume(runState)), errnum);
+    if(XLAL_IS_REAL8_FAIL_NAN(logvolume))
+    {
+	    XLALPrintWarning("Could not compute log volume: %s\n",XLALErrorString(errnum));
+	    logvolume=0.0;
+    }
   }
 
   if(LALInferenceGetProcParamVal(runState->commandLine,"--progress"))
@@ -888,8 +941,8 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   s=initNSintegralState(Nruns,Nlive);
 
   /* Check for an interrupted run */
-  char resumefilename[FILENAME_MAX];
-  sprintf(resumefilename,"%s_resume",outfile);
+  char resumefilename[FILENAME_MAX+10];
+  snprintf(resumefilename,sizeof(resumefilename),"%s_resume",outfile);
   int retcode=1;
   if(LALInferenceGetProcParamVal(runState->commandLine,"--resume")){
 #ifdef HAVE_HDF5
@@ -901,35 +954,11 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
           for(i=0;i<Nlive;i++) logLikelihoods[i]=*(REAL8 *)LALInferenceGetVariable(runState->livePoints[i],"logL");
           iter=s->iteration;
       }
-      /* Install a periodic alarm that will trigger a checkpoint */
-      int sigretcode=0;
-      struct sigaction sa;
-      sa.sa_sigaction=catch_alarm;
-      sa.sa_flags=SA_SIGINFO;
-      sigretcode=sigaction(SIGVTALRM,&sa,NULL);
-      if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint timer!\n");
-      /* Condor sends SIGUSR2 to checkpoint and continue */
-      sigretcode=sigaction(SIGUSR2,&sa,NULL);
-      if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGUSR2.\n");
-      checkpoint_timer.it_interval.tv_sec=30*60; /* Default timer 30 mins */
-      checkpoint_timer.it_interval.tv_usec=0;
-      checkpoint_timer.it_value=checkpoint_timer.it_interval;
-      setitimer(ITIMER_VIRTUAL,&checkpoint_timer,NULL);
-      /* Install the handler for the condor interrupt signal */
-      sa.sa_sigaction=catch_interrupt;
-      sigretcode=sigaction(SIGINT,&sa,NULL);
-      if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGINT.\n");
-      /* Condor sends SIGTERM to vanilla universe jobs to evict them */
-      sigretcode=sigaction(SIGTERM,&sa,NULL);
-      if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGTERM.\n");
-      /* Condor sends SIGTSTP to standard universe jobs to evict them.
-       *I think condor handles this, so didn't add a handler CHECK */
   }
-
   if(retcode!=0)
   {
     if(LALInferenceGetProcParamVal(runState->commandLine,"--resume"))
-        fprintf(stdout,"Unable to open resume file %s. Starting anew.\n",resumefilename);
+        fprintf(stdout,"No resume file %s. Starting a new run.\n",resumefilename);
     /* Sprinkle points */
     LALInferenceSetVariable(runState->algorithmParams,"logLmin",&neginfty);
     int sprinklewarning=0;
@@ -986,8 +1015,8 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   if(!HDFOUTPUT)
   {
       FILE *lout=NULL;
-      char param_list[FILENAME_MAX];
-      sprintf(param_list,"%s_params.txt",outfile);
+      char param_list[FILENAME_MAX+16];
+      snprintf(param_list,sizeof(param_list),"%s_params.txt",outfile);
       lout=fopen(param_list,"w");
       LALInferenceFprintParameterHeaders(lout,runState->livePoints[0]);
       fclose(lout);
@@ -995,6 +1024,11 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   minpos=0;
   threadState->currentParams=currentVars;
   fprintf(stdout,"Starting nested sampling loop!\n");
+  /* Install interrupt handler for resuming */
+  if(LALInferenceGetProcParamVal(runState->commandLine,"--resume"))
+  {
+      install_resume_handler();
+  }
   /* Iterate until termination condition is met */
   do {
     /* Find minimum likelihood sample to replace */
@@ -1142,7 +1176,7 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
 		}
         fclose(fpout);
     
-        char bayesfile[FILENAME_MAX];
+        char bayesfile[FILENAME_MAX+10];
         sprintf(bayesfile,"%s_B.txt",outfile);
         fpout=fopen(bayesfile,"w");
         fprintf(fpout,"%lf %lf %lf %lf\n",logZ-logZnoise,logZ,logZnoise,logLmax);
@@ -1154,14 +1188,14 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
     /* Write HDF5 file */
     if(HDFOUTPUT)
     {
-      
+
       LALH5File *h5file=XLALH5FileOpen(outfile, "w");
       // Create group heirarchy 
-      char runID[256]="";
+      char runID[2048];
       if((ppt=LALInferenceGetProcParamVal(runState->commandLine,"--runid")))
-        snprintf(runID,255,"%s_%s","lalinference_nest",ppt->value);
+        snprintf(runID,sizeof(runID),"%s_%s","lalinference_nest",ppt->value);
       else
-        snprintf(runID,255,"lalinference_nest");
+        snprintf(runID,sizeof(runID),"lalinference_nest");
 
       LALH5File *groupPtr = LALInferenceH5CreateGroupStructure(h5file, "lalinference", runID);
       /* Create run identifier group */
@@ -1174,10 +1208,24 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
       XLALH5FileAddScalarAttribute(groupPtr, "log_max_likelihood", &logLmax , LAL_D_TYPE_CODE);
       XLALH5FileAddScalarAttribute(groupPtr, "number_live_points", &Nlive, LAL_U4_TYPE_CODE);
       XLALH5FileAddScalarAttribute(groupPtr, "log_prior_volume", &logvolume, LAL_D_TYPE_CODE);
+       /* Get the cpu usage */
+      struct tms tms_buffer;
+      if(times(&tms_buffer))
+      {
+        REAL8 execution_time = (REAL8)tms_buffer.tms_utime / (REAL8) sysconf (_SC_CLK_TCK) + LALInferenceGetREAL8Variable(runState->algorithmParams,"cpu_time");
+        XLALH5FileAddScalarAttribute(groupPtr, "cpu_time", &execution_time, LAL_D_TYPE_CODE);
+      }
 
+      LALInferenceVariables *injParams = NULL;
+      if ( (injParams=LALInferencePrintInjectionSample(runState)) )
+      {
+        LALInferenceH5VariablesArrayToDataset(groupPtr, &injParams, 1, "injection_params");
+        LALInferenceClearVariables(injParams);
+        XLALFree(injParams);
+      }
       XLALH5FileClose(h5file);
     }
-  
+
     if(output_array) {
       for(i=0;i<N_output_array;i++){
         LALInferenceClearVariables(output_array[i]);
@@ -1204,8 +1252,8 @@ LALInferenceVariables *LALInferenceComputeAutoCorrelation(LALInferenceRunState *
   /* Single threaded here */
   LALInferenceThreadState *threadState = runState->threads[0];
   ProcessParamsTable *ppt=NULL;
-  char chainfilename[128]="";
-  char acf_file_name[128]="";
+  char chainfilename[2048]="";
+  char acf_file_name[2048]="";
   FILE *chainfile=NULL;
   FILE *acffile=NULL;
   UINT4 i,j;
@@ -1534,7 +1582,7 @@ INT4 LALInferenceNestedSamplingSloppySample(LALInferenceRunState *runState)
     LALInferenceIFOData *data=runState->data;
     REAL8 tmp;
     REAL8 Target=0.3;
-    char tmpName[32];
+    char tmpName[320];
     REAL8 logLold=*(REAL8 *)LALInferenceGetVariable(threadState->currentParams,"logL");
     memset(&oldParams,0,sizeof(oldParams));
     LALInferenceCopyVariables(threadState->currentParams,&oldParams);
